#ifdef HAVE_CONFIG_H
# include "config.h"     
#endif
//C++ INCLUDE
#include <iostream>
//GENERAL DUNE INCLUDE
#include"dune/common/mpihelper.hh" // An initializer of MPI
#include"dune/common/exceptions.hh" // We use exceptions
//
//#include <dune/grid/test/gridcheck.cc>
#include <gridtools.cc>
#include "griddata.cc"
#include "functors.hh"
#include "amr.cc"
#include "f90func.hh"
#include "deformation2.hh"
#include "deformation.hh"
#include "geotools.hh"
#include "myparametrization.hh"

//#include <gridinfo.hh>
//ALUGRID INCLUDE
#include <dune/grid/alugrid.hh> // We use the ALUGrid manager
#include <dune/grid/alugrid/3d/alugrid.hh> // We use the 3D ALUGrid manager
#include <dune/grid/alugrid/3d/alu3dgridfactory.hh> // We use the 3D ALUGrid factory
#include <dune/grid/geometrygrid/grid.hh>
#include <dune/grid/geometrygrid/identity.hh>

int main(int argc, char** argv)
{
  try{
    //Maybe initialize Mpi
    Dune::MPIHelper& helper = Dune::MPIHelper::instance(argc, argv);
    std::cout << "Hello World! This is dune-foo." << std::endl;
    if(Dune::MPIHelper::isFake)
      std::cout<< "This is a sequential program." << std::endl;
    else
      std::cout<<"I am rank "<<helper.rank()<<" of "<<helper.size() 
       <<" processes!"<<std::endl;
    //--------------------//
    //TEST FORTRAN LIBRARY//
    //--------------------//
    double gianni;
    __prova_MOD_ciccio(&gianni);

    //------------------//
    //GENERAL STATEMENTS//
    //------------------//
    //OUTPUT PATH
    std::string outputPath = "outputNEW";

    //---------------//
    //TYPEDEF SECTION//
    //---------------//
    //ALUGRIDCUBE 3D GRID TYPE AND COORDINATE TYPE FOR THAT GRID DEFINITIONS
    typedef Dune::ALUCubeGrid<3,3> GridType;
    //GET GRID DIMENSION
    const int dim = GridType::dimension;
    typedef GridType::ctype ct;
    // DEFORMATION TYPES
    typedef DeformationDiscreteFunction<Dune::FieldVector<ct,dim> > DDF;
    typedef DeformationDiscreteFunction2<Dune::FieldVector<ct,dim>, GridType, Dune::LeafMultipleCodimMultipleGeomTypeMapper<GridType,Dune::MCMGElementLayout>, std::vector<MyParametrization<Dune::FieldVector<ct,dim>,Dune::FieldVector<ct,dim> > > > DDF2;

    //-------------------------------------------------------//
    // BASE GRID DECLARATION AND READING FROM PROSTAR FORMAT //
    //-------------------------------------------------------//
    MPI_Barrier(helper.getCommunicator());
    //DECLARE GRID
    GridType* grid;
    //READ GRID FROM STARCD FILES (.vrt .cel .bnd)
    std::string gridFile="griglie/sevenele";
    grid=readGrid<GridType>(gridFile,helper);

    //-----------------------------------------------------//
    //MAPPER USER DATA INITIALIZATION FROM FUNCTION
    //-----------------------------------------------------//
    //FUNCTION DECLARATION FORM FUNCTORS
    DIST<ct,dim> g;
    //USER DATA CONTAINER DECLARATION(more complex structure will be needed for storing physical fields)
    std::vector<double> c;
    //MAPPER DECLARATION AND FIELD INITIALIZATION BY FUNCTION(element layout for the mapper)
    Dune::MCMGElementLayout<dim> layout;
    //THE CHOICE OF THE PARTITION DEFINE THE NEED FOR COMMUNICATIONS
    Dune::LeafMultipleCodimMultipleGeomTypeMapper<GridType,Dune::MCMGElementLayout> Mapper = LeafVEDataInitFromFunction<0,Dune::InteriorBorder_Partition>(*grid,layout,c,g,helper,"UserInitData",outputPath,"");
    //GHOST ELEMENTS UPDATE - NOT NECESSARY IF INITIALIZED PARTITION IS ALL
    VectorExchange<Dune::LeafMultipleCodimMultipleGeomTypeMapper<GridType,Dune::MCMGElementLayout>,std::vector<double> > dh(Mapper,c,0);
    (*grid).communicate<VectorExchange<Dune::LeafMultipleCodimMultipleGeomTypeMapper<GridType,Dune::MCMGElementLayout>,std::vector<double> > >(dh,Dune::InteriorBorder_All_Interface, Dune::ForwardCommunication);

    //--------------------------//
    //GEOMETRY GRID CONSTRUCTION//
    //--------------------------//
    //DECLARING AN EMPTY CONTAINER FOR ELEMENT PARAMETRIZATION
    std::vector<MyParametrization<Dune::FieldVector<ct,dim>,Dune::FieldVector<ct,dim> > > parametrizations(Mapper.size());
    //CONSTRUCT THE DEFORMATION USING THE BASE GRID, THE MAPPER AND THE CONTAINER FOR STORING PARAMETRIZATIONS 
    DDF2 newDefFun2(*grid, Mapper,parametrizations);
    //IDENTITY DEFORMATION
    DDF defFun;
    //GEOMETRY GEID DECLARATION
    Dune::GeometryGrid<GridType, DDF2> cgrid(*grid,newDefFun2);
    Dune::GeoGrid::ExportParams<GridType,DDF2> exPara;
    geotest(cgrid,newDefFun2,exPara);


        
    // // // CHECK WHAT HAPPENS IN GHOST ELEMENTS
    // // typedef GridType::LeafGridView LeafGridView;
    // // LeafGridView leafview = (*grid).leafView();
    // // typedef LeafGridView::Codim<0>::Partition<Dune::Ghost_Partition>::Iterator GhostLeafIterator;
    // // typedef GhostLeafIterator::Entity::Geometry GhostLeafGeometry;
    // // GhostLeafIterator ghostendit = leafview.end<0,Dune::Ghost_Partition>();
    // // GhostLeafIterator ghostbeginit = leafview.begin<0,Dune::Ghost_Partition>();
    // // for(GhostLeafIterator ig = ghostbeginit; ig != ghostendit; ++ig){
    // //   const GhostLeafGeometry& ggeo = ig->geometry();
    // //   Dune::FieldVector<ct,dim> ggCenterCoords = ggeo.center();
    // //   //verify = data[Mapper.map(*ig)] == temp[Mapper.map(*ig)];
    // //   //std::cout << "GHOSTCHECK rank = " << helper.rank() << " coords = " << ggCenterCoords << "data(coords) = " << verify  << std::endl;
    // //   std::cout << "GHOSTCHECK rank = " << helper.rank() << " coords = " << ggCenterCoords << "data(coords) = "<< c[Mapper.map(*ig)] << std::endl;
    // // }


    std::string gridOutputFile="initialgrid";
    drawGrid(gridOutputFile,outputPath,"",Dune::VTKOptions::ascii,cgrid);


    int counter = 1;
    //FIRST AMR
    std::vector<double> indi(c);
    std::vector<double> indices(c);
    adapting(*grid, Mapper, indi, c, indices, helper, 'F', 0, 10, counter, 0, 0);
    ++counter;
    //    (*grid).loadBalance();//ATTENTION PERSISTENT CONTAINER IS LIKELY NEEDED
    gridOutputFile="firstgrid";
    drawGrid(gridOutputFile,outputPath,"",Dune::VTKOptions::ascii,cgrid);
    
    MPI_Barrier(helper.getCommunicator());

    // //SECOND AMR
    // indi=c;
    // adapting(*grid, Mapper, indi, c, indices, helper, 'F', 0, 10, counter, 0.2, 0);
    // ++counter;
    // //    (*grid).loadBalance();
    // gridOutputFile="secondgrid";
    // drawGrid(gridOutputFile,outputPath,"",Dune::VTKOptions::ascii,*grid);

    // //THIRD AMR
    // indi=c;
    // adapting(*grid, Mapper, indi, c, indices, helper, 'F', 0, 10, counter, 100.0, 1.0);
    // ++counter;
    // //    (*grid).loadBalance();
    // gridOutputFile="thirdgrid";
    // drawGrid(gridOutputFile,outputPath,"",Dune::VTKOptions::ascii,*grid);

    // // //FOURTH AMR
    // // indi=c;
    // // adapting(*grid, Mapper, indi, c, indices, helper, 'F', 0, 10, counter, 100, 0.2);
    // // ++counter;
    // // // (*grid).loadBalance();
    // // gridOutputFile="fourthgrid";
    // // drawGrid(gridOutputFile,outputPath,"",Dune::VTKOptions::ascii,*grid);



    // //exploringMapper(*grid,helper);
    
    // MPI_Barrier(helper.getCommunicator());
    // //completeGridInfo(*grid,0,helper);

    // //std::cout << "I'm " << helper.rank() << " and grid size is " << (*grid).size(0) << std::endl;

    // //visitLeaves(*grid,helper);


    // //test functors
    // Dune::FieldVector<ct,dim> x;
    // x = 1.0;
    // x[0] = 0.0;
    // x[1] = 0.0;
    // DIST<ct,dim> f;
    // std::cout << "In x = " << x << " f(x)= " << f(x) << std::endl;
    // //NEW    

    return 0;
  }
  catch (Dune::Exception &e){
    std::cerr << "Dune reported error: " << e << std::endl;
  }
  catch (...){
    std::cerr << "Unknown exception thrown!" << std::endl;
  }
}
