#ifndef GEOTOOLS_HH
#define GEOTOOLS_HH

#include <dune/grid/geometrygrid/grid.hh>
#include <dune/grid/geometrygrid/iterator.hh>
#include <dune/grid/io/file/vtk/vtkwriter.hh>


template<class DeformedGridType, class EP>
void geotest(DeformedGridType& cgrid, typename EP::CoordFunction& f, EP& exPara)
{
  //BASE GRID GEOMETRY GRID TYPES
  typedef typename EP::HostGrid HG;
  typedef typename EP::CoordFunction CF;
  std::cout << "CGrid size = " << cgrid.size(0) << std::endl;

  //LEAF TYPES
  typedef typename Dune::GeoGrid::template GridFamily<HG,CF,std::allocator<void> >::Traits::template Codim<0>::template Partition<Dune::All_Partition>::LeafIterator LeafIterator;
  typedef typename LeafIterator::Entity::Geometry LeafGeometry;
  typedef typename Dune::GeoGrid::template GridFamily<HG,CF,std::allocator<void> >::Traits::template Partition<Dune::All_Partition>::LeafGridView LeafGridView;
  typedef typename LeafGeometry::Jacobian Jacobian;

  //LEAF INTERSECTION TYPES
  typedef typename Dune::GeoGrid::template GridFamily<HG,CF,std::allocator<void> >::Traits::LeafIntersectionIterator LeafIntersectionIterator;

  

  LeafGridView leafview = cgrid.leafView();

  Dune::VTKWriter<LeafGridView> vtkwriter(leafview);
  vtkwriter.pwrite("testCGrid","outputCGRID","",Dune::VTKOptions::ascii);

  //verifying local parametrization
  LeafIterator endit = leafview.template end<0,Dune::All_Partition>();
  for(LeafIterator it = leafview.template begin<0,Dune::All_Partition>(); it!=endit; ++it)
    {
      const LeafGeometry& geo = it->geometry();
      Dune::FieldVector<double,3> center = geo.center();
      Dune::FieldVector<double,3> localcenter = geo.local(center);
      std::cout << "Element centered in " << center << " and with transposed jacobian in" << localcenter << "= " << geo.jacobianInverseTransposed(localcenter)<<std::endl;

      

      
      //SHOW INTERSECTION NUMBERING CONVENTION
      // LeafIntersectionIterator iend = leafview.iend(*it);
      // for(LeafIntersectionIterator is = leafview.ibegin(*it); is!=iend; ++is)
      // 	{
      // 	  int local = is->indexInInside();
      // 	  Dune::FieldVector<double,3> center = is->geometry().center();
      // 	  int neighbor = 100;
      // 	  if(is->neighbor())
      // 	    neighbor = is->indexInOutside();
      // 	  std::cout << "Intersection centered in " << center << ", index inside = " << local << ", index outside = " << neighbor << std::endl;
	    
      // 	}

      //SHOW LOCAL VERTEX NUMBERING and LOCAL AND GLOBAL POSITION
      // for(int i=0; i<8;++i)
      // 	{
      // 	  Dune::FieldVector<double,3> global = geo.corner(i);
      // 	  Dune::FieldVector<double,3> local = geo.local(global);
      // 	  std::cout << "Corner number " << i << " in " << global << " mapped from " << local << std::endl;
      // 	}
    }

  return;
}



#endif
