#ifndef GRIDDATA
#define GRIDDATA

#include <dune/grid/common/mcmgmapper.hh>
#include <dune/grid/io/file/vtk/vtkwriter.hh>
#include <dune/grid/common/datahandleif.hh>

template <int codim, Dune::PartitionIteratorType ptype, class G, template<int> class L, class V, class F>
Dune::LeafMultipleCodimMultipleGeomTypeMapper<G,L> LeafVEDataInitFromFunction(const G& grid, L<G::dimension>& layout, V& data, const F& function, Dune::MPIHelper& helper,const std::string dataName, const std::string outPath, const std::string extOutPath)
{

  const int dim = G::dimension;
  typedef typename G::ctype ct;
  
  typedef typename G::LeafGridView LeafGridView;
  typedef typename LeafGridView::template Codim<codim>::template Partition<ptype>::Iterator LeafIterator;
  typedef typename LeafIterator::Entity::Geometry LeafGeometry;

  LeafGridView leafGridView = grid.leafView();

  Dune::LeafMultipleCodimMultipleGeomTypeMapper<G,L> Mapper(grid,layout);

  data.resize(Mapper.size());

  LeafIterator endit = leafGridView.template end<codim,ptype>();
  for(LeafIterator it = leafGridView.template begin<codim,ptype>(); it!=endit; ++it)
    {
      const LeafGeometry& geo = it->geometry();
      Dune::FieldVector<ct,dim> global;
      if(codim<dim)
	{
	  global = geo.center();
	}
      else if(codim==dim)
	{
	  global = geo.corner(0);
	}
      data[Mapper.map(*it)] = function(global);
    }

  if(codim != 0 && codim !=dim)
    {
      return Mapper;
    }
  
  Dune::VTKWriter<LeafGridView> vtkwriter(leafGridView);
  if(codim==0)
    vtkwriter.addCellData(data,"data");
  else if(codim==dim)
    vtkwriter.addVertexData(data,"data");
  vtkwriter.pwrite(dataName,outPath,extOutPath,Dune::VTKOptions::ascii);
    
  return Mapper;
};

template <int n, class C>
void ciccio(C& vector)
{
  vector.resize(n);
};


//----------------//
// COMMUNICATIONS //
//----------------//

template <class M, class V>
class VectorExchange : public Dune::CommDataHandleIF<VectorExchange<M,V>, typename V::value_type>
{
public:
  typedef typename V::value_type DataType;

  bool contains(int dim, int codim) const
  {
    return (codim==cd_);
  }

  bool fixedsize(int dim, int codim) const
  {
    return true;
  }

  //customizable expressing a relation between the type of the entity and the number of DataType items to be communicated
  //default is = 1
  template <class EntityType>
  size_t size(EntityType& e) const
  {
    return 1;
  }

  template<class MessageBuffer, class EntityType>
  void gather(MessageBuffer& buff, const EntityType& e) const
  {
    buff.write(c_[mapper_.map(e)]);
  }

  template<class MessageBuffer, class EntityType>
  void scatter(MessageBuffer& buff, const EntityType& e, size_t n) const
  {
    DataType x;
    buff.read(x);
    c_[mapper_.map(e)]=x; //data processing could be more complicated than writing in destination
  }

  VectorExchange(const M& mapper, V& c, int cd) : mapper_(mapper), c_(c), cd_(cd) {}

private:
  const M& mapper_;
  V& c_;
  int cd_;

};

#endif





