#ifndef GRIDTOOLS
#define GRIDTOOLS

/*
It contains a collection of function to handle the grid
*/

#include <iostream>
#include <sstream>
#include "starcdreader.hh" // We use the STAR-CD mesh files reader
//#include <dune/grid/test/gridcheck.cc>
#include <dune/common/mpihelper.hh>
#include <dune/grid/common/grid.hh>
#include <dune/grid/io/file/vtk/vtkwriter.hh>

#include "functors.hh"
#include "gridinfo.hh"


enum partitions {
  Interior, InteriorBorder, All, Ghost
};

template <class GridType>
GridType* readGrid (const std::string& baseName, Dune::MPIHelper& helper)
{
  // read the grid
  std::cout << "Reading the grid" << std::endl;
  MPI_Barrier(helper.getCommunicator());
  GridType* grid=Dune::StarCDReader<GridType>::read(baseName,helper,true);
  
  return grid;
  
} 

template <class GridType>
void completeGridInfo(GridType& grid, int level, Dune::MPIHelper& helper)
{
  // give information about the whole grid, its leaf level and its level level
  std::stringstream ss;
  ss << level;
  std::string strlevel=ss.str();
  MPI_Barrier(helper.getCommunicator());
  Dune::gridinfo(grid,"GridInfo ");
  MPI_Barrier(helper.getCommunicator());
  //Dune::gridleaflist(grid,"GridLeafList ");
  //Dune::gridlevellist(grid,level,"GridLevel"+strlevel+"List ");
  Dune::gridlevellistonfile(grid,level,"GridLevel"+strlevel+"List ","levelList",helper);
  Dune::gridleaflistonfile(grid,"GridLeafList ","leafList",helper);
  MPI_Barrier(helper.getCommunicator());

}


template <class GridType>
void visitLeaves(GridType& grid, Dune::MPIHelper& helper)
{
  const int dim=GridType::dimension; //exporting the dimension of the grid
  typedef typename GridType::ctype ct; // defining the coordinate type

  //Choosing the element iterator by codimension and partition
  const int cd=1;
  const Dune::PartitionIteratorType pittype=Dune::InteriorBorder_Partition;//Interior_Partition, InetriorBorder_Partition, All_Partition, Ghost_Partition

  //Choosing at which level we look at the grid
  typedef typename GridType::LeafGridView GridView;
  GridView gridView=grid.leafView();

  //Choosing the entities and the partition we want to loop on
  typedef typename GridView::template Codim<cd>::template Partition<pittype>::Iterator EntityLeafIterator;
  EntityLeafIterator endit=gridView.template end<cd,pittype>();

  //Loop
  for(EntityLeafIterator it = gridView.template begin<cd,pittype>(); it!=endit; ++it){
    std::cout << "I'm " << helper.rank() << ", entity type= " << it->geometry().type() << " centered in " << it->geometry().center() << " partition= " << Dune::PartitionName(it->partitionType())<< std::endl;
  }


  //write  grid in VTK format
  //Dune::VTKWriter<typename GridType::LeafGridView> vtkwriter(gridView);
  //vtkwriter.write("theGrid",Dune::VTKOptions::ascii);

  return;
}

template <class GridType>
void drawGrid(std::string& fileName,std::string& outPath,std::string extOutPath, Dune::VTK::OutputType type, GridType& grid)
{
  
  typedef typename GridType::LeafGridView GridView;
  GridView gridView=grid.leafView();
  Dune::VTKWriter<typename GridType::LeafGridView> vtkwriter(gridView);
  vtkwriter.pwrite(fileName,outPath,extOutPath,type);
  return;
}




// check macro grid 
//gridcheck(*grid);

//std::cout << " passed." << std::endl;


#endif
