#ifndef MYPARAMETRIZATION_HH
#define MYPARAMETRIZATION_HH

#include <dune/common/function.hh>
#include <dune/common/fvector.hh>

template <class DomainType, class RangeType>
class MyParametrization : public Dune::VirtualFunction<const DomainType, RangeType>
{

public:

  //NO DEFAULT CONSTURCTOR
  MyParametrization(){};

  // contruct the parametrization of a single element using coefficient computed in the DeformationDiscreteFunction constructor
  MyParametrization(std::map<int,std::map<int, std::map<int,RangeType> > >&  GAMMA) : GAMMA_(GAMMA){};

  //copy constructor
  MyParametrization(const MyParametrization & copied){
    GAMMA_ = copied.GAMMA_;
  }

  //operator=
  MyParametrization& operator=(const MyParametrization &rhs){
    if(this!=&rhs){
      GAMMA_ = rhs.GAMMA_;
    }
    return *this;
  }

  //destructor
  ~MyParametrization(){
    //delete &GAMMA_;
  };

  void evaluate(const DomainType& x, RangeType& y) const 
  {
    //initializing y
    for(int i = 0; i <= 2; ++i){
      y[i]=0.0;
    }
    std::map<int,std::map<int, std::map<int,RangeType> > > TEMP = GAMMA_;

    //ELEMENT PARAMETRIZATION
    for(int g = 0; g <= 3; ++g){
      for(int j = 0; j <= 3; ++j){
	for(int k = 0; k <= 3; ++k){
	  for(int i = 0; i <= 2; ++i){
	    y[i] += TEMP[g][j][k][i] *  pow(x[0],g) * pow(x[1],j) * pow(x[2],k);
	  } 
	}
      }
    }

    //y=x;
  };

  //ATTRIBUTES
  std::map<int,std::map<int, std::map<int, RangeType> > > GAMMA_;

};

#endif
